@@ -267,7 +267,7 @@ void AVIFile::Init( int format, int sampleFrequency, int indexType )
 	{
 		indx[ i ] ->wLongsPerEntry = 4;
 		indx[ i ] ->bIndexSubType = 0;
-		indx[ i ] ->bIndexType = AVI_INDEX_OF_INDEXES;
+		indx[ i ] ->bIndexType = KINO_AVI_INDEX_OF_INDEXES;
 		indx[ i ] ->nEntriesInUse = 0;
 		indx[ i ] ->dwReserved[ 0 ] = 0;
 		indx[ i ] ->dwReserved[ 1 ] = 0;
@@ -306,7 +306,7 @@ void AVIFile::Init( int format, int sampleFrequency, int indexType )
     \return 0 if the frame could be found, -1 otherwise
 */
 
-int AVIFile::GetDVFrameInfo( off_t &offset, int &size, int frameNum )
+int AVIFile::GetFrameInfo( off_t &offset, int &size, int frameNum )
 {
 	switch ( index_type )
 	{
@@ -386,21 +386,21 @@ int AVIFile::GetDVFrameInfo( off_t &offset, int &size, int frameNum )
     \return 0 if the frame could be read, -1 otherwise
 */
 
-int AVIFile::GetDVFrame( Frame &frame, int frameNum )
+int AVIFile::GetFrame( Frame *frame, int frameNum )
 {
 	off_t	offset;
 	int	size;
 
-	if ( GetDVFrameInfo( offset, size, frameNum ) != 0 )
+	if ( GetFrameInfo( offset, size, frameNum ) != 0 )
 		return -1;
 	fail_if( lseek( fd, offset, SEEK_SET ) == ( off_t ) - 1 );
-	fail_neg( read( fd, frame.data, size ) );
+	fail_neg( read( fd, frame->data, size ) );
 
 	return 0;
 }
 
 
-int AVIFile::GetTotalFrames() const
+int AVIFile::GetTotalFrames()
 {
 	return mainHdr.dwTotalFrames;
 }
@@ -416,7 +416,7 @@ int AVIFile::GetTotalFrames() const
     \param entry the entry to print
 */
 
-void AVIFile::PrintDirectoryEntryData( const RIFFDirEntry &entry ) const
+void AVIFile::PrintDirectoryEntryData( const RIFFDirEntry &entry )
 {
 	static FOURCC lastStreamType = make_fourcc( "    " );
 
@@ -779,7 +779,7 @@ void AVIFile::FlushIndx( int stream )
 
 	ix[ stream ] ->wLongsPerEntry = 2;
 	ix[ stream ] ->bIndexSubType = 0;
-	ix[ stream ] ->bIndexType = AVI_INDEX_OF_CHUNKS;
+	ix[ stream ] ->bIndexType = KINO_AVI_INDEX_OF_CHUNKS;
 	ix[ stream ] ->nEntriesInUse = 0;
 	ix[ stream ] ->dwChunkId = indx[ stream ] ->dwChunkId;
 	ix[ stream ] ->qwBaseOffset = offset + length;
@@ -1079,7 +1079,7 @@ void AVI1File::Init( int format, int sampleFrequency, int indexType )
 
 /* Write a DV video frame. This is somewhat complex... */
 
-bool AVI1File::WriteFrame( const Frame &frame )
+bool AVI1File::WriteFrame( Frame *frame )
 {
 	int frame_chunk;
 	//    int         junk_chunk;
@@ -1108,15 +1108,15 @@ bool AVI1File::WriteFrame( const Frame &frame )
 	   frame, then add a JUNK chunk which is sized such that we
 	   end up on a 512 bytes boundary. */
 
-	frame_chunk = AddDirectoryEntry( make_fourcc( "00__" ), 0, frame.GetFrameSize(), movi_list );
+	frame_chunk = AddDirectoryEntry( make_fourcc( "00__" ), 0, frame->GetDataLen(), movi_list );
 	if ( ( index_type & AVI_LARGE_INDEX ) && ( streamHdr[ 0 ].dwLength % IX00_INDEX_SIZE ) == 0 )
 	{
 		GetDirectoryEntry( frame_chunk, type, name, length, offset, parent );
 		ix[ 0 ] ->qwBaseOffset = offset - RIFF_HEADERSIZE;
 	}
-	WriteChunk( frame_chunk, frame.data );
-	//    num_blocks = (frame.GetFrameSize() + RIFF_HEADERSIZE) / PADDING_SIZE + 1;
-	//	length = num_blocks * PADDING_SIZE - frame.GetFrameSize() - 2 * RIFF_HEADERSIZE;
+	WriteChunk( frame_chunk, frame->data );
+	//    num_blocks = (frame->GetDataLen() + RIFF_HEADERSIZE) / PADDING_SIZE + 1;
+	//	length = num_blocks * PADDING_SIZE - frame->GetDataLen() - 2 * RIFF_HEADERSIZE;
 	//    junk_chunk = AddDirectoryEntry(make_fourcc("JUNK"), 0, length, movi_list);
 	//    WriteChunk(junk_chunk, g_zeroes);
 
@@ -1428,7 +1428,7 @@ void AVI2File::WriteRIFF()
     \param frame the frame to write
 */
 
-bool AVI2File::WriteFrame( const Frame &frame )
+bool AVI2File::WriteFrame( Frame *frame )
 {
 	int audio_chunk;
 	int frame_chunk;
@@ -1459,7 +1459,8 @@ bool AVI2File::WriteFrame( const Frame &frame )
 
 	/* Write audio data if we have it */
 
-	audio_size = frame.ExtractAudio( soundbuf );
+	assert( !frame->IsHDV() );
+	audio_size = ((DVFrame*)frame)->ExtractAudio( soundbuf );
 	if ( audio_size > 0 )
 	{
 		audio_chunk = AddDirectoryEntry( make_fourcc( "01wb" ), 0, audio_size, movi_list );
@@ -1483,15 +1484,15 @@ bool AVI2File::WriteFrame( const Frame &frame )
 
 	/* Write video data */
 
-	frame_chunk = AddDirectoryEntry( make_fourcc( "00dc" ), 0, frame.GetFrameSize(), movi_list );
+	frame_chunk = AddDirectoryEntry( make_fourcc( "00dc" ), 0, frame->GetDataLen(), movi_list );
 	if ( ( index_type & AVI_LARGE_INDEX ) && ( streamHdr[ 0 ].dwLength % IX00_INDEX_SIZE ) == 0 )
 	{
 		GetDirectoryEntry( frame_chunk, type, name, length, offset, parent );
 		ix[ 0 ] ->qwBaseOffset = offset - RIFF_HEADERSIZE;
 	}
-	WriteChunk( frame_chunk, frame.data );
-	//    num_blocks = (frame.GetFrameSize() + RIFF_HEADERSIZE) / PADDING_SIZE + 1;
-	//	length = num_blocks * PADDING_SIZE - frame.GetFrameSize() - 2 * RIFF_HEADERSIZE;
+	WriteChunk( frame_chunk, frame->data );
+	//    num_blocks = (frame->GetDataLen() + RIFF_HEADERSIZE) / PADDING_SIZE + 1;
+	//	length = num_blocks * PADDING_SIZE - frame->GetDataLen() - 2 * RIFF_HEADERSIZE;
 	//    junk_chunk = AddDirectoryEntry(make_fourcc("JUNK"), 0, length, movi_list);
 	//    WriteChunk(junk_chunk, g_zeroes);
 	if ( index_type & AVI_LARGE_INDEX )
