@@ -71,7 +71,8 @@ DVgrab::DVgrab( int argc, char *argv[] ) :
 		m_no_stop( false ), m_timecode( false ), m_lockstep( false ), m_lockPending( false ),
 		m_lockstep_maxdrops( DEFAULT_LOCKSTEP_MAXDROPS ), m_lockstep_totaldrops( DEFAULT_LOCKSTEP_TOTALDROPS ),
 		m_captureActive( false ), m_avc( 0 ), m_reader( 0 ), m_hdv( false ), m_showstatus( false ),
-		m_isLastTimeCodeSet( false ), m_isLastRecDateSet( false ), m_v4l2( false )
+		m_isLastTimeCodeSet( false ), m_isLastRecDateSet( false ), m_v4l2( false ), m_jvc_p25( false ),
+		m_24p( false ), m_24pa( false )
 {
 	m_frame = 0;
 	m_writer = 0;
@@ -164,18 +165,19 @@ void DVgrab::print_version()
 
 void DVgrab::print_help()
 {
+	print_version();
 	cerr << "Usage: " << m_program_name << " [options] [file] [-]" << endl << endl;
 	cerr << "Use '-' to pipe video to stdout." << endl << endl;
 	cerr << "Options:" << endl << endl;
-	cerr << "  -a, autosplit        start a new file when a new recording is detected" << endl;
+	cerr << "  -a, -autosplit        start a new file when a new recording is detected" << endl;
 	cerr << "  -buffers number      the number of internal frames to buffer [default " << DEFAULT_BUFFERS << "]" << endl;
 	cerr << "  -card number         card number [default automatic]" << endl;
 	cerr << "  -channel number      iso channel number for listening [default " << DEFAULT_CHANNEL << "]" << endl;
-	cerr << "  -cmincutsize num     min file size in MB due to collection split [default " << DEFAULT_CMINCUTSIZE << "]" << endl;
+	cerr << "  -cmincutsize num     min file size in MiB due to collection split [default " << DEFAULT_CMINCUTSIZE << "]" << endl;
 	cerr << "  -csize number        split file when collections of files are about to exceed" << endl;
-	cerr << "                          number MB, 0 = unlimited [default " << DEFAULT_CSIZE << "]" << endl;
+	cerr << "                          number MiB, 0 = unlimited [default " << DEFAULT_CSIZE << "]" << endl;
 	cerr << "  -debug type          display (HDV) debug info, type is one or more of:" << endl;
-	cerr << "                          all,pat,pmt,pids,pid=N,pes,packet,video,sonya1 [default none]" << endl;
+	cerr << "                          all,pat,pmt,pids,pid=N,pes,packet,video,sonya1" << endl;
 	cerr << "  -d, -duration time   total capture duration specified as a SMIL time value:" << endl;
 	cerr << "                          XXX[.Y]h, XXX[.Y]min, XXX[.Y][s], XXXms," << endl;
 	cerr << "                          [[HH:]MM:]SS[.ms], or smpte=[[[HH:]MM:]SS:]FF" << endl;
@@ -189,16 +191,16 @@ void DVgrab::print_help()
 #ifdef HAVE_LIBQUICKTIME
 	cerr << "              qt          QuickTime DV movie" << endl;
 #endif
-	cerr << "              mpeg2/hdv   MPEG-2 transport stream (HDV)" << endl;
+	cerr << "              mpeg2, hdv  MPEG-2 transport stream (HDV)" << endl;
 #if defined(HAVE_LIBJPEG) && defined(HAVE_LIBDV)
-	cerr << "              jpeg/jpg    sequence of JPEG files (DV only)" << endl;
+	cerr << "              jpeg, jpg   sequence of JPEG files (DV only)" << endl;
 #endif
 	cerr << "  -F, -frames number   max number of frames per split" << endl;
 	cerr << "                          0 = unlimited [default " << DEFAULT_FRAMES << "]" << endl;
 	cerr << "  -guid hex            select one of multiple DV devices by its GUID" << endl;
 	cerr << "                          GUID is in hexadecimal; see /sys/bus/ieee1394/devices/" << endl;
 	cerr << "  -h, -help            display this help and exit" << endl;
-	cerr << "  -I, input file       read from file (\"-\" = stdin)" << endl;
+	cerr << "  -I, -input file       read from file (\"-\" = stdin)" << endl;
 	cerr << "  -i, -interactive     go interactive with camera VTR and capture control" << endl;
 #if defined(HAVE_LIBJPEG) && defined(HAVE_LIBDV)
 	cerr << "  -jpeg-deinterlace    deinterlace the output by line doubling the upper field" << endl;
@@ -208,6 +210,8 @@ void DVgrab::print_help()
 	cerr << "  -jpeg-temp name      use name as temporary file output" << endl;
 	cerr << "  -jpeg-width n        scale the output to the specified width (max=2048)" << endl;
 #endif
+	cerr << "  -jvc-p25             remove repeat_first_field flag and set fps to 25 (HDV)" << endl;
+	cerr << "                       (to correct stream from JVC cam recorded in P25 mode)" << endl;
 	cerr << "  -lockstep            align capture to multiple of -frames based on timecode" << endl;
 	cerr << "  -lockstep_maxdrops   max consecutive frame drops before closing file" << endl;
 	cerr << "                          -1 = unlimited [default " << DEFAULT_LOCKSTEP_MAXDROPS << "]" << endl;
@@ -228,6 +232,10 @@ void DVgrab::print_help()
 	cerr << "                          use -input to set device file [default " << DEFAULT_V4L2_DEVICE << "]" << endl;
 #endif
 	cerr << "  -v, -version         display version and exit" << endl;
+#ifdef HAVE_LIBQUICKTIME
+	cerr << "  -24p                 use 24 fps as output rate (Quicktime Only)" << endl;
+	cerr << "  -24pa                remove 2:3:3:2 pulldown for 24p Advanced (Quicktime Only)" << endl;
+#endif
 	cerr << endl;
 	cerr << "Check out the dvgrab website for the latest version, news and other software:" << endl;
 	cerr << "http://www.kinodv.org/" << endl << endl;
@@ -291,7 +299,7 @@ void DVgrab::set_format_from_name( void )
 
 void DVgrab::getargs( int argc, char *argv[] )
 {
-	char *opts = "d:hif:F:I:s:tVv-";
+	const char *opts = "d:hif:F:I:s:tVv-";
 	int optindex = 0;
 	int c;
 	struct option long_opts[] = {
@@ -319,6 +327,7 @@ void DVgrab::getargs( int argc, char *argv[] )
 		{ "jpeg-temp", required_argument, &m_jpeg_usetemp, true },
 		{ "jpeg-width", required_argument, &m_jpeg_width, 0xff },
 #endif
+		{ "jvc-p25", no_argument, &m_jvc_p25, true },
 		{ "lockstep", no_argument, &m_lockstep, true },
 		{ "lockstep_maxdrops", required_argument, &m_lockstep_maxdrops, 0xff },
 		{ "lockstep_totaldrops", required_argument, &m_lockstep_totaldrops, 0xff },
@@ -333,6 +342,10 @@ void DVgrab::getargs( int argc, char *argv[] )
 		{ "timesys", no_argument, &m_timesys, true },
 #ifdef HAVE_LINUX_VIDEODEV2_H
 		{ "v4l2", no_argument, 0, 'V' },
+#endif
+#ifdef HAVE_LIBQUICKTIME
+		{ "24p", no_argument, &m_24p, true },
+		{ "24pa", no_argument, &m_24pa, true },
 #endif
 		{ "version", no_argument, 0, 'v' },
 		{ 0, 0, 0, 0 }
@@ -436,30 +449,42 @@ void DVgrab::getargs( int argc, char *argv[] )
 			print_version();
 			exit( EXIT_SUCCESS );
 			break;
-		case '-':
-			m_raw_pipe = true;
-			break;
 		default:
 			print_usage();
 			exit( EXIT_FAILURE );
 		}
 	}
 
+	if ( optind < argc )
+	{
+		if ( argv[ optind ][0] == '-' )
+		{
+			m_raw_pipe = true;
+			++optind;
+		}
+	}
 	if ( optind < argc )
 	{
 		m_dst_file_name = argv[ optind++ ];
 		set_format_from_name();
 	}
-
 	if ( optind < argc )
 	{
-		cerr << "Too many output file names." << endl;
-		print_usage();
-		exit( EXIT_FAILURE );
+		if ( argv[ optind ][0] == '-' )
+		{
+			m_raw_pipe = true;
+			++optind;
+		}
+		else
+		{
+			cerr << "Too many output file names." << endl;
+			print_usage();
+			exit( EXIT_FAILURE );
+		}
 	}
 
 	if ( m_dst_file_name == NULL && !m_raw_pipe )
-		m_dst_file_name = "dvgrab-";
+		m_dst_file_name = strdup( "dvgrab-" );
 }
 
 void DVgrab::startCapture()
@@ -502,6 +527,8 @@ void DVgrab::startCapture()
 #ifdef HAVE_LIBQUICKTIME
 		case QT_FORMAT:
 			m_writer = new QtHandler();
+			m_writer->SetFilmRate( m_24p );
+			m_writer->SetRemove2332( m_24pa );
 			break;
 #endif
 
@@ -512,7 +539,7 @@ void DVgrab::startCapture()
 #endif
 
 		case MPEG2TS_FORMAT:
-			m_writer = new Mpeg2Handler();
+			m_writer = new Mpeg2Handler( m_jvc_p25 ? MPEG2_JVC_P25 : 0 );
 			break;
 
 		}
@@ -593,7 +620,7 @@ void DVgrab::stopCapture()
 				time( &timesys );
 				localtime_r( &timesys, &recDate );
 			}
-			sendEvent( "\"%s\": %8.2f MB %d frames timecode %2.2d:%2.2d:%2.2d.%2.2d date %4.4d.%2.2d.%2.2d %2.2d:%2.2d:%2.2d",
+			sendEvent( "\"%s\": %8.2f MiB %d frames timecode %2.2d:%2.2d:%2.2d.%2.2d date %4.4d.%2.2d.%2.2d %2.2d:%2.2d:%2.2d",
 			           filename.c_str(), size, frames,
 			           timeCode.hour, timeCode.min, timeCode.sec, timeCode.frame,
 			           recDate.tm_year + 1900, recDate.tm_mon + 1, recDate.tm_mday,
@@ -601,7 +628,7 @@ void DVgrab::stopCapture()
 			         );
 		}
 		else
-			sendEvent( "\"%s\" %8.2f MB %d frames", filename.c_str(), size, frames );
+			sendEvent( "\"%s\" %8.2f MiB %d frames", filename.c_str(), size, frames );
 		sendEvent( "Capture Stopped" );
 
 		if ( m_dropped_frames > 0 )
@@ -715,7 +742,7 @@ void DVgrab::sendCaptureStatus( const char *name, float size, int frames, TimeCo
 	else
 		sprintf( rd_str, "????.??.?? ??:??:??" );
 
-	sendEventParams( 2, 0, "\"%s\": %8.2f MB %5d frames timecode %s date %s%s",
+	sendEventParams( 2, 0, "\"%s\": %8.2f MiB %5d frames timecode %s date %s%s",
 		name, size, frames, tc_str, rd_str, newline ? "\n" : "" );
 }
 
@@ -753,7 +780,7 @@ void DVgrab::writeFrame()
 			localtime_r( &timesys, recDate );
 		}
 
-		if ( m_lockstep && m_lockPending && m_frame->CanStartNewStream() )
+		if ( m_lockstep && m_lockPending && m_frame_count > 0 && m_frame->CanStartNewStream() )
 		{
 			// If a lock is pending due to dropped frames, close the file
 			if ( m_writer->FileIsOpen() )
@@ -875,7 +902,7 @@ void DVgrab::captureThreadRun()
 			sendFrameDroppedStatus( "buffer underrun near",
 				"This error means that the frames could not be written fast enough." );
 			
-			if ( m_lockstep )
+			if ( m_lockstep && m_frame_count > 0 )
 			{
 				if ( m_writer->FileIsOpen() )
 				{
@@ -902,7 +929,7 @@ void DVgrab::captureThreadRun()
 			sendFrameDroppedStatus( "frame dropped",
 				"This error means that the ieee1394 driver received an incomplete frame." );
 
-			if ( m_lockstep )
+			if ( m_lockstep && m_frame_count > 0 )
 			{
 				if ( m_writer->FileIsOpen() )
 				{
