@@ -41,6 +41,7 @@ using std::setfill;
 #include <sys/stat.h>
 #include <assert.h>
 #include <time.h>
+#include <errno.h>
 
 #include "filehandler.h"
 #include "error.h"
@@ -283,6 +284,15 @@ void FileHandler::SetMinColSize( off_t size )
 	minColSize  = size;
 }
 
+void FileHandler::SetFilmRate( bool flag)
+{
+	filmRate = flag;
+}
+
+void FileHandler::SetRemove2332( bool flag)
+{
+	remove2332 = flag;
+}
 
 bool FileHandler::Done()
 {
@@ -459,6 +469,32 @@ TimeSys:
 	return true;
 }
 
+
+static ssize_t writen( int fd, unsigned char *vptr, size_t n )
+{
+	size_t nleft = n;
+	ssize_t nwritten;
+	unsigned char *ptr = vptr;
+
+	while ( nleft > 0 )
+	{
+		if ( ( nwritten = write( fd, ptr, nleft ) ) <= 0 )
+		{
+			if ( errno == EINTR )
+			{
+				nwritten = 0;
+			}
+			else
+			{
+				return -1;
+			}
+		}
+		nleft -= nwritten;
+		ptr += nwritten;
+	}
+	return n;
+}
+
 /***************************************************************************/
 
 
@@ -497,7 +533,7 @@ bool RawHandler::Create( const string& filename )
 
 int RawHandler::Write( Frame *frame )
 {
-	int result = write( fd, frame->data, frame->GetDataLen() );
+	int result = writen( fd, frame->data, frame->GetDataLen() );
 	return result;
 }
 
@@ -822,6 +858,7 @@ inline void QtHandler::DeinterlaceStereo16( void* pInput, int iBytes,
 int QtHandler::Write( Frame *f )
 {
 	assert( !f->IsHDV() );
+	int result = 0;
 	DVFrame *frame = (DVFrame*)f;
 
 	if ( ! isFullyInitialized )
@@ -840,8 +877,15 @@ int QtHandler::Write( Frame *f )
 			channels = 0;
 		}
 
-		quicktime_set_video( fd, 1, 720, frame->IsPAL() ? 576 : 480,
-		                     frame->GetFrameRate(), QUICKTIME_DV );
+		if ( filmRate || remove2332 )
+		{
+			quicktime_set_video( fd, 1, 720, 480, 24, QUICKTIME_DV );
+		}
+		else
+		{
+			quicktime_set_video( fd, 1, 720, frame->IsPAL() ? 576 : 480,
+			                     frame->GetFrameRate(), QUICKTIME_DV );
+		}
 
 		if ( channels > 0 )
 		{
@@ -868,9 +912,28 @@ int QtHandler::Write( Frame *f )
 		isFullyInitialized = true;
 	}
 
-	int result = quicktime_write_frame( fd, const_cast<unsigned char*>( frame->data ),
-	                                    frame->GetDataLen(), 0 );
+	if ( remove2332 )
+	{
+		const int frameOffset = 3;
+		TimeCode tc;
+		frame->GetTimeCode( tc );
 
+		if ( tc.frame + frameOffset % 5 )
+		{
+			result = quicktime_write_frame( fd, const_cast<unsigned char*>( frame->data ),
+			                                frame->GetFrameSize(), 0 );
+		}
+		else
+		{
+			result = 1;
+		}
+	}
+	else
+	{
+		result = quicktime_write_frame( fd, const_cast<unsigned char*>( frame->data ),
+		                                frame->GetFrameSize(), 0 );
+	}
+ 
 	if ( channels > 0 )
 	{
 		AudioInfo audio;
@@ -1201,14 +1264,21 @@ int JPEGHandler::Close( void )
 /***************************************************************************/
 
 
-Mpeg2Handler::Mpeg2Handler( const string& ext ) :
-	fd( -1 ), waitingForRecordingDate( true ), bufferLen( 0 ), totalFrames( 0 )
+Mpeg2Handler::Mpeg2Handler( unsigned char flags, const string& ext ) :
+	fd( -1 ), waitingForRecordingDate( true ), bufferLen( 0 ), totalFrames( 0 ),
+	writerFlags( flags ), firstPayloadEntry( NULL )
 {
 	extension = ext;
 }
 
 Mpeg2Handler::~Mpeg2Handler()
 {
+	PayloadList *next;
+	while ( firstPayloadEntry != NULL ) {
+	    next = firstPayloadEntry->next;
+	    free( firstPayloadEntry );
+	    firstPayloadEntry = next;
+	}	
 	Close();
 }
 
@@ -1261,12 +1331,19 @@ int Mpeg2Handler::Write( Frame *frame )
 	// Write any buffered data first.
 	if ( bufferLen > 0 )
 	{
-		if ( 0 > ( result = write( fd, buffer, bufferLen ) ) )
+		if ( frame->CouldBeJVCP25() && ( writerFlags & MPEG2_JVC_P25 ) )
+			result = writeJVCP25( buffer, bufferLen );
+		else
+			result = writen( fd, buffer, bufferLen );
+		if ( 0 > result )
 			return result;
 		bufferLen = 0;
 	}
 
-	result = write( fd, frame->data, frame->GetDataLen() );
+	if ( frame->CouldBeJVCP25() && ( writerFlags & MPEG2_JVC_P25 ) )
+		result = writeJVCP25( frame->data, frame->GetDataLen() );
+	else
+		result = writen( fd, frame->data, frame->GetDataLen() );
 
 	if ( 0 <= result )
 		totalFrames++;
@@ -1305,3 +1382,203 @@ int Mpeg2Handler::GetFrame( Frame *frame, int frameNum )
 {
 	return -1;
 }
+
+static inline
+int CorrectP25( unsigned char *data, unsigned char len, unsigned char *state )
+{
+	int i;
+
+	for (i=0; i<len; i++)
+	{
+		switch (*state)
+		{
+		/* seek for start code prefix 0x00 0x00 0x01 */
+		case 0:
+		case 1:
+			if ( 0x00 == data[i] )
+				(*state)++;
+			else
+				*state = 0;
+			break;
+
+		case 2:
+			if ( 0x01 == data[i] )
+				*state = 3;
+			else
+				*state = 0;
+			break;
+
+		/* seek for start code values of sequence_header or */
+		/* picture coding extension header */
+		case 3:
+			if ( SEQUENCE_HEADER_CODE_VALUE == data[i] ) /* Sequence Header? => ignore next 3 bytes */
+				*state = 4;
+			else if ( EXTENSION_START_CODE_VALUE == data[i] ) /* Extension Header? */
+				*state = 14;
+			else
+				*state = 0;
+			break;
+
+		/* read over three more bytes */
+		case 4:
+		case 5:
+		case 6:
+		case 15:
+		case 16:
+			(*state)++;
+			break;
+
+		case 14:
+			if ( PICTURE_CODING_EXTENSION_ID_VALUE  == (data[i] >> 4) )   /* Picture Coding extension ?*/
+				*state = 15;
+			else
+				*state = 0;
+			break;
+
+		/* the eights bit has to be changed for both parameters */
+		case 7:
+			/* change 50 fps to 25 fps  */
+				/* least significant 4 bits */
+			/* 0110 = 50 fps	    */
+			/* 0011 = 25 fps	    */
+	
+			/* works only with value of 50fps */
+			/* data[i] ^= 0x05; */
+			data[i] = ( data[i] & 0xF0 ) | 0x03;
+			*state = 0;
+			break;
+
+		case 17:
+			/* unset repeat_first_field flag */
+			/* value |=  0x02 flag set	 */
+			/* value &= ~0x02 flag unset	 */
+			data[i] &= ~0x02;	
+			*state = 0;
+			break;
+
+		default:
+			/* undefined state */
+			return -1;
+		} /* switch */
+    } /* for */
+    return 0;
+}
+
+
+void Mpeg2Handler::ProcessPayload( unsigned char *packet, unsigned int pid, unsigned char len )
+{
+	static PayloadList	*current = NULL;
+	
+	if ( NULL != firstPayloadEntry ) /* look for the struct for the current pid */
+		for ( current=firstPayloadEntry; ( ( NULL!=current ) && ( pid!=current->pid ) ); current=current->next );
+	
+	if (NULL == current)
+	{
+		current = ( PayloadList* ) malloc( sizeof(PayloadList) );
+		if ( NULL == current )
+		{
+			fprintf( stderr, "Error allocating memory!\n" );
+			exit( 1 );
+		}
+	
+		if ( NULL == firstPayloadEntry )
+			firstPayloadEntry = current;
+	
+		current->pid = pid;
+		current->state = 0;
+		current->next = NULL;
+	} /* if */
+	CorrectP25( packet, len, &(current->state) );
+}
+
+
+void Mpeg2Handler::ProcessTSPacket( unsigned char *packet )
+{
+	unsigned int pid;
+
+	pid = ((packet[1] & 0x1F) << 8 ) | packet[2];
+	if (0x1FFF == pid)
+		return; /* throw NULL packet away */
+
+	switch (packet[3] & 0x30)
+	{
+	case 0x00:		/* reserved */
+	case 0x20:		/* adaptation field without payload */
+		break;
+
+	case 0x10:		/* payload only */
+		ProcessPayload( &packet[4], pid, 188-4 );
+		break;
+
+	case 0x30:		/* adaptation field before payload */
+		if (183 > packet[4])
+		/* max_length of ts_packet
+		   - 4 bytes header
+		   - 1 byte adaptation-length info
+		   - adaptation-length */
+		ProcessPayload( &packet[188 - 4 - 1 - packet[4]], pid, 188 - 4 - 1 - packet[4] );
+	    break;
+	} /* switch */
+}
+
+
+int Mpeg2Handler::writeJVCP25( unsigned char *data, int len )
+{
+	static unsigned char state = 0, ts_packet[188], rest_length;
+	int i;
+	unsigned char next_possible_start_position = 0;
+	
+	for ( i = 0; i < len; i++ )
+	{
+		switch (state)
+		{
+		/* seek for HDV_PACKET_MARKER of transport packet */
+		case 0:
+			if ( HDV_PACKET_MARKER == data[i] )
+			{
+				ts_packet[0] = data[i];
+				rest_length = 187;
+				state = 1;
+			}
+			break;
+
+		case 1:
+			if (0 < rest_length)
+			{
+				ts_packet[ 188 - rest_length ] = data[i];
+				rest_length--;
+				if ( ! next_possible_start_position )
+				if ( HDV_PACKET_MARKER == data[i] )
+					next_possible_start_position = i;
+			}
+			else 
+			{
+				/* 188 byte seen */
+				if ( HDV_PACKET_MARKER == data[i] )
+				{
+					/* last 188 bytes were ts packet */
+					ProcessTSPacket( ts_packet );
+					writen( fd, ts_packet, 188 );
+					ts_packet[0] = data[i];
+					rest_length = 187;
+				}
+				else
+				{
+					/* last 188 bytes are not a ts packet */
+					/* scan again beginning with the next possible start of ts_packet */
+					state = 0;
+					/* write unchanged first bytes up to next_possible_start_position */
+					writen( fd, ts_packet, next_possible_start_position + 1 );
+					writeJVCP25( &ts_packet[ next_possible_start_position ],
+						188 - next_possible_start_position);
+					ts_packet[ 188 - rest_length ] = data[i];
+				} /* if */
+			} /* if */
+			break;
+
+		default:
+			return -1; /* undefined state */
+		} /* switch */
+    } /* for */
+    return 0;
+}
