@@ -295,6 +295,8 @@ iec61883Reader::iec61883Reader( int p, int c, int bufSize,
 		m_resetHandlerData( data )
 {
 	m_handle = NULL;
+	m_iec61883_mpeg2 = NULL;
+	m_iec61883_dv = NULL;
 }
 
 
@@ -407,13 +409,13 @@ bool iec61883Reader::Open()
 
 		if ( isHDV )
 		{
-			m_iec61883.mpeg2 = iec61883_mpeg2_recv_init( m_handle, Mpeg2HandlerProxy, this );
-			success = ( m_iec61883.mpeg2 != NULL );
+			m_iec61883_mpeg2 = iec61883_mpeg2_recv_init( m_handle, Mpeg2HandlerProxy, this );
+			success = ( m_iec61883_mpeg2 != NULL );
 		}
 		else
 		{
-			m_iec61883.dv = iec61883_dv_fb_init( m_handle, DvHandlerProxy, this );
-			success = ( m_iec61883.dv != NULL );
+			m_iec61883_dv = iec61883_dv_fb_init( m_handle, DvHandlerProxy, this );
+			success = ( m_iec61883_dv != NULL );
 		}
 	}
 	catch ( string exc )
@@ -432,14 +434,15 @@ bool iec61883Reader::Open()
 
 void iec61883Reader::Close()
 {
-	if ( m_iec61883.ref != NULL )
+	if ( m_iec61883_dv != NULL )
 	{
-		if ( isHDV )
-			iec61883_mpeg2_close( m_iec61883.mpeg2 );
-		else
-			iec61883_dv_fb_close( m_iec61883.dv );
-
-		m_iec61883.dv = NULL;
+		iec61883_dv_fb_close( m_iec61883_dv );
+		m_iec61883_dv = NULL;
+	}
+	else if ( m_iec61883_mpeg2 != NULL )
+	{
+		iec61883_mpeg2_close( m_iec61883_mpeg2 );
+		m_iec61883_mpeg2 = NULL;
 	}
 }
 
@@ -451,9 +454,9 @@ bool iec61883Reader::StartReceive()
 	try
 	{
 		if ( isHDV )
-			fail_neg( iec61883_mpeg2_recv_start( m_iec61883.mpeg2, channel ) );
+			fail_neg( iec61883_mpeg2_recv_start( m_iec61883_mpeg2, channel ) );
 		else
-			fail_neg( iec61883_dv_fb_start( m_iec61883.dv, channel ) );
+			fail_neg( iec61883_dv_fb_start( m_iec61883_dv, channel ) );
 		success = true;
 	}
 	catch ( string exc )
@@ -467,14 +470,13 @@ bool iec61883Reader::StartReceive()
 
 void iec61883Reader::StopReceive()
 {
-	if ( m_iec61883.ref != NULL )
+	if ( m_iec61883_dv != NULL )
 	{
-		if ( isHDV )
-			iec61883_mpeg2_close( m_iec61883.mpeg2 );
-		else
-			iec61883_dv_fb_close( m_iec61883.dv );
-
-		m_iec61883.ref = NULL;
+		iec61883_dv_fb_stop( m_iec61883_dv );
+	}
+	else if ( m_iec61883_mpeg2 != NULL )
+	{
+		iec61883_mpeg2_recv_stop( m_iec61883_mpeg2 );
 	}
 }
 
@@ -1085,6 +1087,7 @@ void* pipeReader::Thread()
 	pthread_mutex_lock( &mutex );
 	if ( currentFrame ) outFrames.push_back( currentFrame );
 	currentFrame = NULL;
+	outFrames.push_back( currentFrame );
 	TriggerAction( );
 	pthread_mutex_unlock( &mutex );
 	return NULL;
