@@ -32,7 +32,6 @@
 #include <string>
 #include <iostream>
 using std::cout;
-using std::cerr;
 using std::endl;
 
 // C includes
@@ -52,6 +51,7 @@ using std::endl;
 
 #include "io.h"
 #include "dvgrab.h"
+#include "error.h"
 
 bool g_done = false;
 
@@ -70,7 +70,7 @@ int rt_raisepri (int pri)
 	 */
 	if ( sysconf( _SC_PRIORITY_SCHEDULING ) == -1)
 	{
-		cerr << "Warning: RR-scheduler not available, disabling." << endl;
+		sendEvent( "Warning: RR-scheduler not available, disabling." );
 		return -1;
 	}
 	else 
@@ -79,7 +79,7 @@ int rt_raisepri (int pri)
 		scp.sched_priority = sched_get_priority_max( SCHED_RR ) - pri;
 		if ( sched_setscheduler( 0, SCHED_RR, &scp ) < 0 )
 		{
-			cerr << "Warning: Cannot set RR-scheduler" << endl;
+			sendEvent( "Warning: Cannot set RR-scheduler" );
 			return -1;
 		}
 	}
@@ -108,7 +108,7 @@ int main( int argc, char *argv[] )
 
 #if _POSIX_MEMLOCK > 0
 		if ( mlockall( MCL_CURRENT | MCL_FUTURE ) == -1 )
-			cerr << "Warning: Cannot disable swapping" << endl;
+			sendEvent( "Warning: Cannot disable swapping" );
 #endif
 
 		if ( dvgrab.isInteractive() )
