@@ -12,7 +12,8 @@ extern "C"
 	int raw1394_get_num_ports( void );
 	raw1394handle_t raw1394_open( int port );
 	void raw1394_close( raw1394handle_t handle );
-	int discoverAVC( int * port, octlet_t guid );
+	int discoverAVC( int * port, octlet_t* guid );
+	void reset_bus( int port );
 
 #ifdef __cplusplus
 }
